﻿using UnityEngine;
using System.Collections;

public class Asteroid : MonoBehaviour {

	public float m_size = 1.0f;
	public float m_minYOffset = 0.1f;
	public int m_asteroidScore = 1;
	public Camera m_camera;
	public Transform m_transform;
	public Rigidbody2D m_rigidbody;
	public Lives m_lives;
	public Score m_score;
	public GameObject m_collision;
	public AttachedExplosion m_explosion;
	
	private Vector3 m_startPos;
	private float m_sleepTimer = 0.0f;
	private bool m_waveEnding = false;
	private State m_state = State.Finished;
	private Wave.Asteroid m_data;
	
	private enum State
	{
		Sleeping,
		Running,
		Finished
	}

	void Start()
	{
		Finish();
	}
	
	void Update ()
	{
		if(State.Sleeping == m_state)
		{
			// No gravity if asleep.
			m_sleepTimer -= Time.deltaTime;
			if(0 >= m_sleepTimer)
			{
				m_state = State.Running;
				m_collision.SetActive(true);
				m_rigidbody.gravityScale = Random.Range(m_data.m_minAcc, m_data.m_maxAcc);
				m_rigidbody.drag = Random.Range(m_data.m_minDrag, m_data.m_maxDrag);
			}
		}
		else if(State.Running == m_state)
		{
			Vector3 minPos = m_camera.ScreenToWorldPoint (new Vector3(0.0f, m_minYOffset*Screen.height, 0.0f));
			Vector3 pos = m_transform.position;
			if(pos.y < minPos.y - m_size)
			{
				m_explosion.Fire (pos);

				if(m_waveEnding)
				{
					Finish ();
				}
				else
				{
					ResetPos ();
				}
			}
		}
	}

	void OnCollisionEnter2D()
	{
		m_explosion.Fire (m_transform.position);

		m_lives.Kill ();

		Finish ();
	}
	
	void ResetPos ()
	{
		Vector3 minPos = m_camera.ScreenToWorldPoint (new Vector3 (0.0f, 0.0f));
		minPos += new Vector3 (m_size, -m_size, 0.0f);
		Vector3 maxPos = m_camera.ScreenToWorldPoint (new Vector3 (Screen.width, Screen.height));
		maxPos += new Vector3 (-m_size, m_size, 0.0f);

		m_state = State.Sleeping;
		m_collision.SetActive (false);
		m_sleepTimer = Random.Range (m_data.m_minWait, m_data.m_maxWait);
		m_transform.position = new Vector3(Random.Range(minPos.x, maxPos.x), maxPos.y, 0.0f);
		m_transform.Rotate (0.0f, 0.0f, Random.Range(0, 360.0f));
		float size = Random.Range (m_data.m_minScale, m_data.m_maxScale);
		m_transform.localScale = new Vector3 (size, size, 0.0f);
		m_rigidbody.gravityScale = 0.0f;
		m_rigidbody.velocity = Vector2.zero;
		m_rigidbody.angularVelocity = Random.Range (-m_data.m_maxAngularVel, m_data.m_maxAngularVel);
	}

	void WaveEnding()
	{
		m_waveEnding = true;
	}

	public bool IsFinished { get { return State.Finished == m_state; } }
	
	public void Reset(Wave.Asteroid data)
	{
		m_data = data;
		m_waveEnding = false;
		m_explosion.Hide ();
		ResetPos ();
	}

	public void Shoot()
	{
		m_score.AddScore(m_asteroidScore);
		Finish ();
	}

	public void Finish()
	{
		m_state = State.Finished;
		m_collision.SetActive (false);
		Vector3 minPos = m_camera.ScreenToWorldPoint (new Vector3 (0.0f, 0.0f));
		minPos.z = 0.0f;
		minPos += new Vector3 (m_size, -m_size, 0.0f);
		m_transform.position = minPos;
		m_rigidbody.velocity = Vector2.zero;
		m_rigidbody.angularVelocity = 0.0f;
		m_rigidbody.gravityScale = 0.0f;
	}
}
