﻿using UnityEngine;
using System.Collections;
using InControl;

public class GameController : MonoBehaviour {

	public Lives m_lives;
	public AudioSource m_gameOverSound;
	public AudioSource m_highscoreSound;
	public Popup m_popup;
	public float m_deadWaitTime = 2.0f;
	public Ship m_ship;

	private State m_state = State.Intro;
	private int m_highscore = -1;
	private float m_timer = -1.0f;

	private enum State
	{
		Intro,
		Running,
		Dying,
		GameOver
	}

	void Update()
	{
		if(State.Intro == m_state)
		{
			UpdateIntro();
		}
		else if(State.Running == m_state)
		{
			UpdateRunning();
		}
		else if(State.Dying == m_state)
		{
			UpdateDying();
		}
		else if(State.GameOver == m_state)
		{
			UpdateGameOver();
		}
	}

	void UpdateIntro()
	{
		if(InputUtils.AnyButtonPressed())
		{
			m_state = State.Running;
			Utils.SendMessageToAll("ResetGame");
		}
	}

	void UpdateRunning()
	{
		if(m_lives.IsDead)
		{
			m_state = State.Dying;
			m_timer = m_deadWaitTime;
			if(m_highscore < Score.m_score)
			{
				m_highscore = Score.m_score;
				m_highscoreSound.Play();
				m_popup.SetText("You scored " + m_highscore + "!\n\nNew High Score!");
			}
			else
			{
				m_popup.SetText("You scored " + m_highscore + ".\n\nThe high score is " + m_highscore + ".");
			}
			Utils.SendMessageToAll("GameOver");
			m_gameOverSound.Play();
		}
	}

	void UpdateDying()
	{
		m_timer -= Time.deltaTime;
		if(0 >= m_timer)
		{
			m_popup.EnableButton();
			m_state = State.GameOver;
		}
	}

	void UpdateGameOver()
	{
		if(InputUtils.AnyButtonPressed())
		{
			m_state = State.Running;
			m_ship.ResetShip();
			Utils.SendMessageToAll("ResetGame");
		}
	}
}
