﻿using UnityEngine;
using System.Collections;
using InControl;

public class Arm : MonoBehaviour
{
	public float m_turretSpeed = 2.0f;
	public float m_armSpeed = 0.01f;
	public float m_startExtenion = 0.0f;
	public Transform m_armT;
	public ArmSegment[] m_armSegments;

	private float m_extension = 0.0f;
	private bool m_gameOver = false;
	private Quaternion m_startAngle = Quaternion.identity;

	void Awake()
	{
		m_startAngle = m_armT.rotation;
	}

	InputDevice GetInputDevice()
	{
		if(1 < InputManager.Devices.Count)
		{
			return InputManager.Devices[1];
		}
		
		return null;
	}
	
	void FixedUpdate ()
	{
		if(m_gameOver)
		{
			return;
		}

		InputDevice device = GetInputDevice();
		if(null != device)
		{
			float stickX, stickY = 0.0f;
			InputUtils.GetStickXY(device, out stickX, out stickY);

			float dir = GetDir(stickX, stickY);
			m_armT.Rotate(Vector3.forward, m_turretSpeed*dir);

			// Arm extension
			InputControl rightTrigger = device.GetControl (InputControlType.RightTrigger);
			InputControl leftTrigger = device.GetControl (InputControlType.LeftTrigger);
			m_extension += m_armSpeed*(rightTrigger.Value - leftTrigger.Value);

			m_extension = Mathf.Clamp01(m_extension);
		}

		SetExtension (m_extension);
	}

	private void SetExtension(float extension)
	{
		m_extension = extension;

		// Set arm extension
		for(int i = 0; i < m_armSegments.Length; ++i)
		{
			m_armSegments[i].SetExtention(m_extension);
		}
	}

	private float GetDir(float x, float y)
	{
		if(0 != x || 0 != y)
		{
			Vector3 toTouch = new Vector3(x, y, 0.0f);
			
			// Direction
			toTouch.Normalize();
			float touchDotRight = Vector3.Dot(toTouch, -m_armT.right);

			float dir = -2*touchDotRight;
			return Mathf.Clamp(dir, -1, 1);
		}
		else
		{
			return 0;
		}
	}
	
	void GameOver()
	{
		m_gameOver = true;
	}
	
	void ResetGame()
	{
		m_gameOver = false;
		m_armT.rotation = m_startAngle;
		SetExtension (m_startExtenion);
	}
}
