﻿using UnityEngine;
using System.Collections;
using InControl;

public class Movement : MonoBehaviour
{
	public Transform m_shipT;
	public Rigidbody2D m_rigidBody;
	public Camera m_camera;
	public float m_speed = 1.5f;
	public float m_minYOffset = 0.1f;

	private bool m_gameOver = false;
	private Vector3 m_startPos = Vector3.zero;

	void Start()
	{
		m_startPos = m_shipT.position;
	}

	InputDevice GetInputDevice()
	{
		if(0 < InputManager.Devices.Count)
		{
			return InputManager.Devices[0];
		}

		return null;
	}

	void FixedUpdate ()
	{
		if(m_gameOver)
		{
			return;
		}

		InputDevice device = GetInputDevice();
		if(null != device)
		{
			float stickX, stickY = 0.0f;
			InputUtils.GetStickXY(device, out stickX, out stickY);
			m_rigidBody.AddForce(m_speed*(new Vector2(stickX, stickY)));
		}

		// Limit to edge of the screen.
		LimitPos ();
	}

	private void LimitPos()
	{
		Vector3 minPos = m_camera.ScreenToWorldPoint (new Vector3(0.0f, m_minYOffset*Screen.height, 0.0f));
		Vector3 maxPos = m_camera.ScreenToWorldPoint (new Vector3(Screen.width, Screen.height, 0.0f));

		// X
		Vector3 worldPos = m_shipT.position;
		if(worldPos.x < minPos.x)
		{
			worldPos.x = minPos.x;
			LimitX(worldPos);
		}

		if(worldPos.x > maxPos.x)
		{
			worldPos.x = maxPos.x;
			LimitX(worldPos);
		}

		// Y
		if(worldPos.y < minPos.y)
		{
			worldPos.y = minPos.y;
			LimitY(worldPos);
		}
		
		if(worldPos.y > maxPos.y)
		{
			worldPos.y = maxPos.y;
			LimitY(worldPos);
		}
	}

	private void LimitX(Vector3 pos)
	{
		m_shipT.position = pos;
		Vector2 vel = m_rigidBody.velocity;
		vel.x = 0;
		m_rigidBody.velocity = vel;
	}
	
	private void LimitY(Vector3 pos)
	{
		m_shipT.position = pos;
		Vector2 vel = m_rigidBody.velocity;
		vel.y = 0;
		m_rigidBody.velocity = vel;
	}

	void GameOver()
	{
		m_gameOver = true;
		m_rigidBody.velocity = Vector2.zero;
	}

	void ResetGame()
	{
		m_gameOver = false;
		m_shipT.position = m_startPos;
	}
}
