﻿using UnityEngine;
using System.Collections;
using InControl;

public class Turret : MonoBehaviour {

	public float m_turretSpeed = 2.0f;
	public Transform m_turretT;
	public GameObject m_missilePrefab;
	public GameObject m_missileExplosionPrefab;
	public int m_numMissiles = 8;
	public float m_missileSpeed = 10.0f;
	
	private bool m_gameOver = false;
	private Quaternion m_startAngle = Quaternion.identity;
	private GameObject[] m_missiles;
	private GameObject[] m_missileExplosions;
	
	void Awake()
	{
		m_startAngle = m_turretT.rotation;
	}

	void OnDestroy()
	{
		for(int i = 0; i < m_numMissiles; ++i)
		{
			GameObject.Destroy(m_missiles[i]);
			GameObject.Destroy(m_missileExplosions[i]);
		}
	}

	void Start()
	{
		m_missiles = new GameObject[m_numMissiles];
		for(int i = 0; i < m_numMissiles; ++i)
		{
			GameObject missile = Instantiate(m_missilePrefab) as GameObject;
			Transform missileT = missile.transform;
			missileT.parent = null;
			missileT.position = Vector2.zero;
			missile.SetActive(false);
			m_missiles[i] = missile;
		}

		m_missileExplosions = new GameObject[m_numMissiles];
		for(int i = 0; i < m_numMissiles; ++i)
		{
			GameObject explosion = Instantiate(m_missileExplosionPrefab) as GameObject;
			Transform missileT = explosion.transform;
			missileT.parent = null;
			missileT.position = Vector2.zero;
			explosion.SetActive(false);
			m_missileExplosions[i] = explosion;
			m_missiles[i].GetComponent<Missile>().m_explosion = explosion;
		}
	}
	
	InputDevice GetInputDevice()
	{
		if(2 < InputManager.Devices.Count)
		{
			return InputManager.Devices[2];
		}
		
		return null;
	}

	void Fire()
	{
		for(int i = 0; i < m_numMissiles; ++i)
		{
			if(!m_missiles[i].activeSelf)
			{
				Transform missileT = m_missiles[i].transform;
				missileT.position = m_turretT.position;
				missileT.rotation = m_turretT.rotation;
				m_missiles[i].SetActive(true);
				m_missiles[i].rigidbody2D.velocity = -1.0f*m_missileSpeed*m_turretT.up;
				return;
			}
		}
	}

	void Update ()
	{
		if(m_gameOver)
		{
			return;
		}

		InputDevice device = GetInputDevice();
		if(null != device)
		{
			if(device.GetControl(InputControlType.Action1).WasPressed || device.GetControl(InputControlType.RightTrigger).WasPressed)
			{
				Fire();
			}
		}
		else
		{
			if(Input.GetKeyDown("space"))
			{
				Fire();
			}
		}
	}
	
	void FixedUpdate ()
	{
		if(m_gameOver)
		{
			return;
		}
		
		InputDevice device = GetInputDevice();
		if(null != device)
		{
			float stickX, stickY = 0.0f;
			InputUtils.GetStickXY(device, out stickX, out stickY);
			
			float dir = GetDir(stickX, stickY);
			m_turretT.Rotate(Vector3.forward, m_turretSpeed*dir);
		}
	}
	
	private float GetDir(float x, float y)
	{
		if(0 != x || 0 != y)
		{
			Vector3 toTouch = new Vector3(x, y, 0.0f);
			
			// Direction
			toTouch.Normalize();
			float touchDotRight = Vector3.Dot(toTouch, -m_turretT.right);
			
			float dir = -2*touchDotRight;
			return Mathf.Clamp(dir, -1, 1);
		}
		else
		{
			return 0;
		}
	}
	
	void GameOver()
	{
		m_gameOver = true;
	}
	
	void ResetGame()
	{
		m_gameOver = false;
		m_turretT.rotation = m_startAngle;
	}
}
