﻿using UnityEngine;
using System.Collections;

public class WaveController : MonoBehaviour {

	[System.Serializable]
	public class ProgressWave
	{
		public int m_minDifficulty;
		public int m_maxDifficulty;
	}

	public ProgressWave[] m_progressWaves;
	public Wave[] m_waves;
	public Asteroid[] m_asteroids;
	public Target[] m_targets;

	private int m_progress = 0;
	private Wave m_currentWave = null;
	private float m_waveTimer = 0.0f;
	private State m_state = State.Waiting;

	public int GetProgress() { return m_progress; }

	private enum State
	{
		Waiting,
		Running,
		Ending
	}

	void Update()
	{
		if(State.Running == m_state)
		{
			UpdateRunning();
		}
		else if(State.Ending == m_state)
		{
			UpdateEnding();
		}
	}

	void UpdateRunning()
	{
		m_waveTimer -= Time.deltaTime;

		if(0 >= m_waveTimer)
		{
			m_state = State.Ending;
			Utils.SendMessageToAll("WaveEnding");
		}
	}

	void UpdateEnding()
	{
		// Wait until all the asteroids and targets are finished.
		if(AllTargetsFinished() && AllAsteroidsFinished())
		{
			m_progress++;
			m_progress = Mathf.Min (m_progress, m_progressWaves.Length-1);
			ChooseWave ();
		}
	}

	bool AllTargetsFinished()
	{
		for(int i = 0; i < m_targets.Length; ++i)
		{
			if(!m_targets[i].IsFinished)
			{
				return false;
			}
		}

		return true;
	}
	
	bool AllAsteroidsFinished()
	{
		for(int i = 0; i < m_asteroids.Length; ++i)
		{
			if(!m_asteroids[i].IsFinished)
			{
				return false;
			}
		}
		
		return true;
	}

	void ChooseWave()
	{
		m_state = State.Running;

		int minDifficulty = m_progressWaves [m_progress].m_minDifficulty;
		int maxDifficulty = m_progressWaves [m_progress].m_maxDifficulty;

		ArrayList possibleWaves = new ArrayList ();
		for(int i = 0; i < m_waves.Length; ++i)
		{
			float difficulty = m_waves[i].m_difficulty;
			if(minDifficulty <= difficulty && difficulty <= maxDifficulty)
			{
				possibleWaves.Add(m_waves[i]);
			}
		}

		if(0 < possibleWaves.Count)
		{
			int random = Random.Range (0, possibleWaves.Count);
			m_currentWave = possibleWaves[random] as Wave;
		}
		else
		{
			Debug.LogError("No possible wave found!");

			// Just use the last wave.
			m_currentWave = m_waves[m_waves.Length-1];
		}

		m_waveTimer = m_currentWave.m_duration;

		// Setup Asteroids
		for(int i = 0; i < m_currentWave.m_asteroids.Length; ++i)
		{
			Wave.Asteroid asteroidData = m_currentWave.m_asteroids[i];
			m_asteroids[i].Reset(asteroidData);
		}

		for(int i = m_currentWave.m_asteroids.Length; i < m_asteroids.Length; ++i)
		{
			m_asteroids[i].Finish();
		}

		// Setup targets
		for(int i = 0; i < m_currentWave.m_targets.Length; ++i)
		{
			Wave.Target targetData = m_currentWave.m_targets[i];
			m_targets[i].Reset(targetData);
		}
		
		for(int i = m_currentWave.m_targets.Length; i < m_targets.Length; ++i)
		{
			m_targets[i].Finish();
		}
	}

	void ResetGame()
	{
		m_progress = 0;
		ChooseWave ();
	}
}
